/*
INDUCTIVE LOAD CENTER ILC(prototype)
 Charles Prichard
 12 December 2015

 ILC: An important device to prevent inverter overloading caused by multiple inductive loads.

 LTSPICE simulation using discrete component models was created featuring a back-to-back retriggerable
 monostable concoction. Lack of interest in the invention gives cause to creating the programmable
 prototype version. This will actually lead to an easier-to-implement piece of hardware, and bypass
 a development phase to accommodate anyone wanting to program new applications for this circuit which
 can possibly be ported to the retriggerable monostable version.

 Program logic for prioritization is less than expected. Note there is memory for more program features.
 Arduino Micro provides a small 34-pin footprint to drop into a board with driver transistors, relays,
 and sensor conditioning, or my dummy sense implementation.

 I heard via NPR story a few days ago of renewable system in Wales UK, where use of too many devices would
 overload the inverter. This story was interesting as the location is the site where Led Zeppelin was inspired
 to write "Stairway to Heaven," and an expensive wind turbine is used. Lack of a device to limit inductive loads
 to prevent overloading, provides greater assurance there is no device like the ILC in the world.
 
 I recently replaced two cables from 24-volt battery bus to inverter to improve performance. Now, voltage drop is 
 only 1-volt into the inverter when pumping water, or the septic. The pumps take less time, operating more
 efficiently. I burned my finger on one of the battery type connectors to #4 aluminum wire. Wire is now #2 copper,
 and connectors are crimped. The water pumps noticeably faster to repressurize, and the septic requires about ten seconds.
 Before changing cables, low voltage was indicated most of the time. Now, it does not happen unless battery voltage is 
 less than 23 volts. The aluminum cables and connectors were deteriating.

 Target: Arduino Micro

 This code intends to read four priority request (PRQ) inputs labeled 6,7,9,10 on the Arduino board.
 Based on input a single prioritized output on pins 2,3,4,5 is articulated with prioritization by order of processing.
 A predefined timeslice is used to allocate inverter power by driving four relays to switch line voltage
 via the prioritized output and a driver transistor.

 In cases where a sensor is driving the PRQ input signal, as the sensor is polled affecting prq_State, power will be unswitched.
 Testing prq_State allows a control loop test to fail, resulting in switching off the driven relay ahead of a timeout 
 which will otherwise do so.

 I decided to read composite PRQ inputs during the sync loop, reducing response time to sensor input in the timeslice loop.

 A sync pulse and 1sec delay is generated between timeslice loops.

 Switch on the board is reset. Use of pin8 as switch pin input is programmed to expire an active timeslice. 
 
 It may be possible to sense battery voltage and sunlight, asserting power for automated air purification and other use.
 This would be the beginning of creating additional features via programming.


 */
// int DC0_PRQ_update(unsigned long);
                                         // Global Constants
const char* global_hello = "ARDUINO MICRO";
                                                        
const int swPin = 10;                    // switch pin reference                                   
const int led = 13;                      // LED reference 

const boolean DC0 = true;         // True if Dummy Card option implemented. False if not implemented.
const boolean DC1 = true;
const boolean DC2 = true;
const boolean DC3 = true;


                                 
const int prq0_Pin = 2;                  // references for PRQ0-PRQ3 input pins
const int prq1_Pin = 3;
const int prq2_Pin = 4;
const int prq3_Pin = 5;                                  

const int p0_Pout = 6;                   // references for P0-P3 output pins
const int p1_Pout = 7;
const int p2_Pout = 8;
const int p3_Pout = 9;

const int sync_Pout = 11;                // sync output pin reference
                                   

                                         // 600000 = 1 minute in milliseconds
const long delay_Sync = 100;             // 100 millisecond sync pulse

const long tsl_P0 = 10000;               // P0 timeslice (10 seconds, adjust for desired delay.) 
const long tsl_P1 = 100;                 // P1 timeslice
const long tsl_P2 = 600;                 // P2 timeslice
const long tsl_P3 = 600;                 // P3 timeslice

const long delay_tsl_P0 = 10000;               // P0 timeslice (10 seconds, adjust for desired delay.)
const long delay_tsl_P1 = 100;                 // P1 timeslice
const long delay_tsl_P2 = 600;                 // P2 timeslice
const long delay_tsl_P3 = 600;                 // P3 timeslice

                                         // Global Variables
int charVal;
String instr = ""; 
char ch;                     

int prq0_State = LOW;                    // loop PRQ_0 state
int prq1_State = LOW;                    // loop PRQ_1 state
int prq2_State = LOW;                    // loop PRQ_2 state
int prq3_State = LOW;                    // loop PRQ_3 state

int DC_P0 = LOW;
int DC_P1 = LOW;
int DC_P2 = LOW;
int DC_P3 = LOW;

unsigned long DC0_ltime = 0;       // beginning loop time 
unsigned long DC1_ltime = 0;
unsigned long DC2_ltime = 0;
unsigned long DC3_ltime = 0;

unsigned long pTime = 0;                 // present time
unsigned long lTime = 0;                 // beginning loop time
long Timer = 0;                          // loop timer

// int swState = 0;                      // current switch state
long timeslice = 0;


void setup() {
  
  Serial.begin(9600);
  splash(); 
  
  pinMode(led, OUTPUT);                  // set the LED pin as output
  pinMode(sync_Pout, OUTPUT);            // set the syncOut pin as output

  pinMode(p0_Pout, OUTPUT);              // set the prioritized output pins as outputs
  pinMode(p1_Pout, OUTPUT);
  pinMode(p2_Pout, OUTPUT);
  pinMode(p3_Pout, OUTPUT);

  // pinMode(swPin, INPUT);              // set the switch pin as input
  
  pinMode(prq0_Pin, INPUT);              // set the priority request pins as input
  pinMode(prq1_Pin, INPUT);
  pinMode(prq2_Pin, INPUT);
  pinMode(prq3_Pin, INPUT);
  
  digitalWrite(led, LOW);                // Initialize LED LOW
}

void splash(){

  while (!Serial.available()){}
        
  digitalWrite(sync_Pout, HIGH);       // generate a sync pulse
  Timer = 0;
  lTime = millis();
  do {                                 // This delay can be interrupted.
  
    pTime = millis();                  // Is also useful for input sense
    Timer = pTime - lTime;
  }
  while (Timer < 1000);
  digitalWrite(sync_Pout, LOW);
  
  Serial.println(global_hello);

}

int DCPRQ_update(unsigned long DC_ltime, long delay_PRQ ) {

  unsigned long ptime = 0;
  long Timer = 0;

  ptime = millis();
  Timer = ptime - DC_ltime;

  if (Timer >= delay_PRQ){return HIGH;}
  
  return LOW;
  
}

void loop() {
                                         // declare and assign loop variables
  int DC_ptr = 0;                  
  int prqState = LOW;                    // Used to determine if demand is fulfilled.
  int pOut = LOW;                        // Used to set and clear output assertion.
  int prq_Pin = LOW;                     // Used to remember which input to test for state change.
  int ptr = LOW;
  char* buf[32];
                                            
  digitalWrite(sync_Pout, HIGH);         // generate a sync pulse
  Timer = 0;
  lTime = millis();  
  do {                                   // This delay can be interrupted. 
                                         // Read the PRQ inputs
  // prq0_State = HIGH;                  // prq0_State = digitalRead(prq0_Pin);
  if (!DC_P0) {prq0_State = digitalRead(prq0_Pin);}
  else{prq0_State = DCPRQ_update(DC0_ltime,delay_tsl_P0);}
  if (!DC_P1) {prq1_State = digitalRead(prq1_Pin);}
  else{prq1_State = DCPRQ_update(DC1_ltime,delay_tsl_P1);}
  if (!DC_P2) {prq2_State = digitalRead(prq2_Pin);}
    else{prq2_State = DCPRQ_update(DC2_ltime,delay_tsl_P2);}  
  if (!DC_P3) {prq3_State = digitalRead(prq3_Pin);}
    else{prq3_State = DCPRQ_update(DC3_ltime,delay_tsl_P3);}  // prq0_State = digitalRead(prq0_Pin);    // TEST here verifies PRQ inputs are HIGH unless pulled LOW.
  //prq1_State = LOW;                      // prq1_State = digitalRead(prq1_Pin);
  //prq2_State = LOW;                      // prq2_State = digitalRead(prq2_Pin);
  //prq3_State = LOW;                      // prq3_State = digitalRead(prq3_Pin);                            

    
  pTime = millis();                    // Is also useful for input sense
    Timer = pTime - lTime;
  }
  while (Timer < delay_Sync);
  digitalWrite(sync_Pout, LOW);

  if (prq0_State = HIGH) {               //Can program a switch to reverse priority...
    timeslice = tsl_P0;
    pOut = p0_Pout;
  if (DC0){
    DC_ptr = 1;
    prq_Pin = LOW;  
  } else { 
    prq_Pin = prq0_Pin;
  }
  } else {
    if (prq1_State = HIGH) {
      timeslice = tsl_P1;
      pOut = p1_Pout;
      if (DC1){
        DC_ptr = 2;
        prq_Pin = LOW;
      } else {
        prq_Pin = prq0_Pin;
      }     
    } else {
              if (prq2_State = HIGH) {
        timeslice = tsl_P2;
        pOut = p2_Pout;
        if (DC2){
          DC_ptr = 3;
          prq_Pin = LOW;
        } else {
          prq_Pin = prq0_Pin;
        }
          } else {
          if (prq3_State = HIGH) {
            timeslice = tsl_P3;
            pOut = p3_Pout;
            if (DC3){
              DC_ptr = 4;
              prq_Pin = LOW;
            }
            else {
              prq_Pin = prq0_Pin;
            }
          }                        
        }
     }
   }
                                                       
   if ((prq0_State = LOW) && (prq1_State = LOW) && (prq2_State = LOW) && (prq3_State = LOW)){
     timeslice = 100;                                    // default timeslice if no PRQ inputs HIGH 
     pOut = led;   
   }
  
  
  digitalWrite (pOut, HIGH);                             // Generate a timeslice
  digitalWrite (led, HIGH);
  Timer = 0;
  
  lTime = millis();
  do {
    
      if (Serial.available()){ charVal = Serial.read();}
      if (charVal){
        digitalWrite(led, HIGH);            // set LED on
        // delay(1);                  // wait
        ptr++;
        instr.concat(char(charVal)); 
        Serial.write(charVal);
        Serial.write(32);               // space to obviate TX emanation
        charVal = 0;
        digitalWrite(led, LOW);             // set LED off
        // delay(1);                  // wait
      }
    
  if ((pOut != led) && (prq_Pin)){prqState = digitalRead(prq_Pin);}   // Test prq_Pin for state change                          
                             // prqState = HIGH; 
    if ((pOut != led) && (prqState = LOW)){break;}     // Sensed input ends pOut assertion
    
  pTime = millis();
    Timer = pTime - lTime;
  }
  while (Timer < timeslice);
  if(DC_ptr){
    if(DC_ptr = 1){DC0_ltime = millis();}         // Reset DC_PRQ delay timer
    if(DC_ptr = 2){DC1_ltime = millis();}
    if(DC_ptr = 3){DC2_ltime = millis();}
    if(DC_ptr = 4){DC3_ltime = millis();}
  DC_ptr = 0;   
  }
   
  digitalWrite(pOut, LOW);                                //Timeout ends pOut assertion.
  digitalWrite(led, LOW);
  
  charVal = 0;
  ptr++;
  instr.concat(char(charVal));  
  Serial.println(instr);
  instr = "";
  
}                                                         //LOOP


